/**************************************************************
 *
 *	vecarith.h
 *
 *	Routines for AltiVec-based giant arithmetic.
 *
 *  This package is part of ongoing research in the
 *	Advanced Computation Group, Apple Computer.
 *	
 *	c. 1999 Apple Computer, Inc.
 *	All Rights Reserved.
 *
 *
 *************************************************************/

/*
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#if __MWERKS__
#include <altivec.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif
									

// pResult = pVec2 * pVec1
void VecMult(						vector unsigned long *pVec1,
									unsigned long	len1,
									vector unsigned long *pVec2,
									unsigned long	len2,
									vector unsigned long *pResult);								
// pResult = pVec1*pVec1
void VecSquare(						vector unsigned long *pVec1,
									unsigned long	len1,
									vector unsigned long *pResult);
																		
// pResult = A0:A1 * B0:B1
void mult2by2(						vector unsigned long  *pA0,
									vector unsigned long  *pA1,
									vector unsigned long  *pB0,
									vector unsigned long  *pB1,
									vector unsigned long  *pResult);

// pDest = pMultiplier * pVectors
void VecMult1(						vector unsigned long *pMultiplier,
									unsigned long	vecCount,
									vector unsigned long *pVectors,
									vector unsigned long *pDest);


///////////////////////////////////			
// assembly routines (in .s file)
///////////////////////////////////			

// vecs *= multiplier
unsigned long MultVecsByULong(		vector unsigned long *vecs,
									unsigned long vecCount,
									register unsigned long multiplier);
// vecs += addDigit
unsigned long AddULongToVecs(		vector unsigned long *vecs,
									unsigned long vecCount,
									register unsigned long addDigit);

// pResult = pVec1 + pVec2
unsigned long addvecs(			vector unsigned long *pVec1,
								unsigned long	len1,
								vector unsigned long *pVec2,
								unsigned long	len2,
								vector unsigned long *pResult);

// pResult = pVec2 - pVec1
void subvecs(					vector unsigned long *pVec1,
								unsigned long	len1,
								vector unsigned long *pVec2,
								unsigned long	len2,
								vector unsigned long *pResult);
								
#ifdef __cplusplus
}
#endif
								
