//////////
//
//	File:		QTReadWriteJPEG.h
//
//	Contains:	Sample code for compressing and decompressing JPEG images.
//
//	Copyright:	 1996-1998 by Apple Computer, Inc., all rights reserved.
//
//	Change History (most recent first):
//
//	   <1>	 	04/27/98	rtm		first file
//	   
//////////
/*
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//////////
//
// header files
//
//////////

#include <FixMath.h>
#include <ImageCompression.h>
#include <Movies.h>
#include <QuickTimeComponents.h>
#include <StandardFile.h>


#ifndef PASCAL_RTN	
#define PASCAL_RTN pascal
#endif


//////////
//
// constants
//
//////////

#define kImageFileCreator		FOUR_CHAR_CODE('ogle')
#define kBufferSize				codecMinimumDataSize		// data unload buffer size

#define kSaveImagePrompt		"Save compressed image as:"
#define kSaveImageFileName		"Untitled.jpg"
#define kWindowTitle			"Photo - JPEG"

// JFIF (JPEG) markers
enum {
	kSOFMarker					= 0xC0,						// start Of Frame N
	kSOFMarker2					= 0xC2,
	kDHTMarker					= 0xC4,						// DHT Marker
	kDACMarker					= 0xCC,						// DAC

	kRSTOMarker					= 0xD0,						// RSTO marker
	kSOIMarker					= 0xD8, 					// image start marker
	kEOIMarker					= 0xD9,						// image end marker
	kSOSMarker					= 0xDA,						// SOS marker
	kDQTMarker					= 0xDB,						// DQT marker

	kAPPOMarker					= 0xE0,						// APPO marker
	kCommentMarker				= 0xFE,						// comment marker

	kStartMarker				= 0xFF						// marker loacted after this byte
};


//////////
//
// data types
//
//////////

// a record to hold info about the JPEG data while we're loading it from disk
struct DLDataRec
{	
	long						fRefNum;					// ref number of current file
	long						fFileLength;			
	Ptr							fOrigPtr;					// address of start of buffer
	Ptr							fEndPtr;					// address of end of buffer
};
typedef struct DLDataRec DLDataRec;
typedef DLDataRec *DLDataPtr, **DLDataHnd;


//////////
//
// function prototypes
//
//////////

void							QTJPEG_PromptUserForJPEGFileAndDisplay (void);
OSErr							QTJPEG_ReadJPEG (FSSpec theFSSpec, CGrafPtr *theGWorld);
OSErr							QTJPEG_ConvertJPEG (Handle image, CGrafPtr *newWorld);
OSErr 							QTJPEG_ReadJPEGHeader (short theRefNum, ImageDescriptionHandle theDesc, Rect *theRect);
UInt8							QTJPEG_FindNextMarker (long theRefNum);
OSErr 							QTJPEG_HandleAPPOMarker (UInt8 theMarker, long theRefNum, ImageDescriptionHandle theDesc, Boolean *readingExtension);
OSErr 							QTJPEG_HandleSOFMarker (long theRefNum, ImageDescriptionHandle theDesc, Boolean readingExtension);
void 							QTJPEG_HandleSOSMarker (long theRefNum);
UInt8  							QTJPEG_ReadByte (short theRefNum);
UInt16 							QTJPEG_ReadWord (short theRefNum);
void 							QTJPEG_SkipLength (long theRefNum);
OSErr 							QTJPEG_NewJPEGWorld (GWorldPtr *theWorld, short theDepth, Rect theRect);
OSErr 							QTJPEG_SaveJPEG (GWorldPtr theWorld);
PASCAL_RTN OSErr 				QTJPEG_DataUnloadProc (Ptr theData, long theBytesNeeded, long theRefCon);
PASCAL_RTN OSErr 				QTJPEG_DataLoadingProc (Ptr *theData, long theBytesNeeded, long theRefCon);
